
#include "lexer.h"
#include "parser.h"

#include <qfile.h>
#include <qtextstream.h>
#include <iostream>

#define __DEBUG         1

int main( int /*argc*/, char* argv[] )
{
    QFile f( argv[1] );
    if( f.open(IO_ReadOnly) ){
        QTextStream stream( &f );
        QString source = stream.read();
        f.close();

        CppSupport::Lexer lexer;

        // stl
        lexer.addSkipWord( "__STL_BEGIN_NAMESPACE" );
        lexer.addSkipWord( "__STL_END_NAMESPACE" );
        lexer.addSkipWord( "__STL_NULL_TMPL_ARGS" );
        lexer.addSkipWord( "__GC_CONST" );

        // qt
        lexer.addSkipWord( "Q_TEMPLATE_EXTERN" );
        lexer.addSkipWord( "Q_EXPORT" );
        lexer.addSkipWord( "QM_EXPORT_NETWORK" );
        lexer.addSkipWord( "QMAC_PASCAL" );
        lexer.addSkipWord( "QT_STATIC_CONST" );
        lexer.addSkipWord( "Q_OVERRIDE", CppSupport::SkipWordAndArguments );
        lexer.addSkipWord( "Q_ENUMS", CppSupport::SkipWordAndArguments );
        lexer.addSkipWord( "Q_PROPERTY", CppSupport::SkipWordAndArguments );
        lexer.addSkipWord( "Q_INLINE_TEMPLATES" );
        lexer.addSkipWord( "Q_TYPENAME" );

        lexer.setSource( source );

        CppSupport::Parser parser( &lexer );
        parser.setFileName( argv[1] );

        parser.parseTranslationUnit();
        qDebug( "\n%d errors", parser.errors() );

#ifdef __DEBUG
        if( parser.errors() == 0 ){
            qDebug( "-------------------------------------------------------------------------------" );
            qDebug( "%s", parser.dom().toString().latin1() );
            qDebug( "-------------------------------------------------------------------------------" );
        }
#endif
    }

    return 0;
}

