
#ifndef PARSER_H
#define PARSER_H

#include <qstring.h>
#include <qobject.h>
#include <qdom.h>

namespace CppSupport{

class Lexer;
class Token;

class Parser: public QObject{
    Q_OBJECT
public:
    Parser( Lexer* lexer );
    virtual ~Parser();

    QString fileName() const;
    void setFileName( const QString& fileName );

    int errors() const;

    virtual void resetErrors();
    virtual void reportError( const QString& msg, const Token& token );
    virtual void reportWarning( const QString& msg, const Token& token);
    virtual void syntaxError();

    const QDomDocument& dom() const { return *m_dom; }

public /*rules*/ :

    bool parseTranslationUnit();

    bool parseDefinition( QDomElement& );
    bool parseLinkageSpecification( QDomElement& );
    bool parseLinkageBody( QDomElement& );
    bool parseNamespace( QDomElement& );
    bool parseUsing( QDomElement& );
    bool parseUsingDirective( QDomElement& );
    bool parseTypedef( QDomElement& );
    bool parseAsmDefinition( QDomElement& );
    bool parseTemplateDeclaration( QDomElement& );
    bool parseDeclaration( QDomElement& );
    bool parseNestedNameSpecifier( QDomElement& );
    bool parseName( QDomElement& );
    bool parseName2( QDomElement& ); /*internal*/
    bool parseOperatorFunctionId( QDomElement& );
    bool parseTemplateArgumentList( QDomElement& );
    bool parseOperator( QDomElement& );
    bool parseCvQualify( QDomElement& );
    bool parseSimpleTypeSpecifier( QDomElement& );
    bool parsePtrOperator( QDomElement& );
    bool parseTemplateArgument( QDomElement& );
    bool parseTypeSpecifier( QDomElement& );
    bool parseDeclarator( QDomElement& );
    bool parseTemplateParameterList( QDomElement& );
    bool parseTemplateParameter( QDomElement& );
    bool parseDeclHead( QDomElement& );
    bool parseIntegralDeclaration( QDomElement& );
    bool parseIntegralDeclHead( QDomElement& );
    bool parseOtherDeclaration( QDomElement& );
    bool parseConstDeclaration( QDomElement& );
    bool parseConstructorDeclaration( QDomElement& );
    bool parseStorageClassSpecifier( QDomElement& );
    bool parseFunctionSpecifier( QDomElement& );
    bool parseConstantExpression( QDomElement& );
    bool parseInitDeclaratorList( QDomElement& );
    bool parseInitDeclarator( QDomElement& );
    bool parseFunctionBody( QDomElement& );
    bool parseParameterDeclarationClause( QDomElement& );
    bool parseCtorInitializer( QDomElement& );
    bool parsePtrToMember( QDomElement& );
    bool parseEnumSpecifier( QDomElement& );
    bool parseClassSpecifier( QDomElement& );
    bool parseElaboratedTypeSpecifier( QDomElement& );
    bool parseDeclaratorId( QDomElement& );
    bool parseExceptionSpecification( QDomElement& );
    bool parseCommaExpression( QDomElement& );
    bool parseEnumeratorList( QDomElement& );
    bool parseEnumerator( QDomElement& );
    bool parseTypeParameter( QDomElement& );
    bool parseParameterDeclaration( QDomElement& );
    bool parseTypeId( QDomElement& );
    bool parseAbstractDeclarator( QDomElement& );
    bool parseParameterDeclarationList( QDomElement& );
    bool parseAssignmentExpression( QDomElement& );
    bool parseClassHead( QDomElement& );
    bool parseMemberSpecificationList( QDomElement& );
    bool parseMemberSpecification( QDomElement& );
    bool parseAccessSpecifier( QDomElement& );
    bool parseBaseClause( QDomElement& );
    bool parseTypeIdList( QDomElement& );
    bool parseMemInitializerList( QDomElement& );
    bool parseMemInitializer( QDomElement& );
    bool parseInitializer( QDomElement& );
    bool parseBaseSpecifierList( QDomElement& );
    bool parseBaseSpecifier( QDomElement& );
    bool parseExpressionList( QDomElement& );
    bool parseInitializerClause( QDomElement& );
    bool parseMemInitializerId( QDomElement& );

protected:
    bool skipUntil( int token );
    bool skip( int l, int r );
    bool isConstructorDecl();

private:
    Lexer* lex;
    QDomDocument* m_dom;
    QDomElement m_translationUnit;
    QString m_fileName;
    int m_errors;
};

inline QString Parser::fileName() const
{
    return m_fileName;
}

inline int Parser::errors() const
{
    return m_errors;
}

inline void Parser::resetErrors()
{
    m_errors = 0;
}


}

#endif
